#!/usr/bin/env python3
import logging
import os
from pathlib import Path
from typing import Optional

import click

from clin.clinfile import calculate_scope
from clin.config import ConfigurationError, load_config
from clin.nakadi import Nakadi, NakadiError
from clin.processor import Processor, ProcessingError
from clin.utils import configure_logging, pretty_yaml, pretty_json
from clin.yamlops import YamlLoader, load_manifest, load_yaml, YamlError

DEFAULT_YAML_LOADER = YamlLoader()


@click.group()
def cli():
    """Manage Nakadi resources"""

    pass


@cli.command("apply")
@click.option("-e", "--env", required=True, type=str, help="Nakadi environment")
@click.option("-t", "--token", required=False, type=str, help="Bearer token")
@click.option(
    "-X",
    "--execute",
    is_flag=True,
    default=False,
    help="Execute updates (default - false)",
)
@click.option(
    "-v",
    "--verbose",
    is_flag=True,
    default=False,
    help="Verbose output (default - false)",
)
@click.option(
    "-d", "--show-diff", is_flag=True, default=False, help="Show diff (default - false)"
)
@click.option(
    "-p",
    "--show-payload",
    is_flag=True,
    default=False,
    help="Show Nakadi payload (default - false)",
)
@click.argument("file", type=click.Path(exists=True, dir_okay=False, readable=True))
def apply(
    env: str,
    token: Optional[str],
    execute: bool,
    verbose: bool,
    show_diff: bool,
    show_payload: bool,
    file: str,
):
    """Create or update Nakadi resource from single yaml manifest file\n
     Values to fill {{VARIABLES}} are taken from system environment"""
    configure_logging(verbose)

    try:
        config = load_config()
        kind, spec = load_manifest(Path(file), DEFAULT_YAML_LOADER, os.environ)
        processor = Processor(config, token, execute, show_diff, show_payload)
        processor.apply(env, kind, spec)

    except (ProcessingError, NakadiError, ConfigurationError, YamlError) as ex:
        logging.error(ex)
        exit(-1)

    except Exception as ex:
        logging.exception(ex)
        exit(-1)


@cli.command("process")
@click.option("-t", "--token", required=False, type=str, help="Bearer token")
@click.option("-X", "--execute", is_flag=True, default=False, help="Execute updates")
@click.option(
    "-v",
    "--verbose",
    is_flag=True,
    default=False,
    help="Verbose output (default - false)",
)
@click.option(
    "-d", "--show-diff", is_flag=True, default=False, help="Show diff (default - false)"
)
@click.option(
    "-p",
    "--show-payload",
    is_flag=True,
    default=False,
    help="Show Nakadi payload (default - false)",
)
@click.argument("file", type=click.Path(exists=True, dir_okay=False, readable=True))
def process(
    token: Optional[str],
    execute: bool,
    verbose: bool,
    show_diff: bool,
    show_payload: bool,
    file: str,
):
    """Create or update multiple Nakadi resources from clin file"""
    configure_logging(verbose)

    try:
        config = load_config()
        processor = Processor(config, token, execute, show_diff, show_payload)
        file_path: Path = Path(file)
        master = load_yaml(file_path, DEFAULT_YAML_LOADER, os.environ)
        scope = calculate_scope(master, file_path.parent, DEFAULT_YAML_LOADER)
        event_types = [et for et in scope if et.kind == "event-type"]
        subscriptions = [sub for sub in scope if sub.kind == "subscription"]

        for task in event_types + subscriptions:
            logging.debug(
                "[%s] applying file %s to %s environment",
                task.id,
                task.path,
                task.target,
            )
            processor.apply(task.target, task.kind, task.spec)

    except (ProcessingError, ConfigurationError, YamlError) as ex:
        logging.error(ex)
        exit(-1)

    except Exception as ex:
        logging.exception(ex)
        exit(-1)


@cli.command("dump")
@click.option("-e", "--env", required=True, type=str, help="Nakadi environment")
@click.option("-t", "--token", required=False, type=str, help="Bearer token")
@click.option(
    "-o",
    "--output",
    default="yaml",
    type=click.Choice(["yaml", "json"]),
    help="Output format",
)
@click.option(
    "-v",
    "--verbose",
    is_flag=True,
    default=False,
    help="Verbose output (default - false)",
)
@click.argument("event_type", type=str)
def dump(env: str, token: Optional[str], output: str, verbose: bool, event_type: str):
    """Print manifest of existing Nakadi event type"""
    configure_logging(verbose)

    try:
        config = load_config()
        if env not in config.environments:
            logging.error(f"Environment not found in configuration: {env}")
            exit(-1)

        nakadi = Nakadi(config.environments[env].nakadi_url, token)
        et = nakadi.get_event_type(event_type)
        if not et:
            logging.error("Event type not found in Nakadi %s: %s", env, event_type)
            exit(-1)

        if output.lower() == "yaml":
            logging.info(pretty_yaml(et.to_spec()))
        elif output.lower() == "json":
            logging.info(pretty_json(et.to_spec()))
        else:
            logging.error("Invalid output format: %s", output)
            exit(-1)

    except (NakadiError, ConfigurationError) as ex:
        logging.error(ex)
        exit(-1)

    except Exception as ex:
        logging.exception(ex)
        exit(-1)


if __name__ == "__main__":
    cli()



from __future__ import annotations

from dataclasses import dataclass
from typing import List, Dict, Optional, Union

from clin.utils import ensure_flat_list


@dataclass
class AllowedTenants:
    admins: List[str]
    writers: List[str]
    readers: List[str]


@dataclass
class Auth:
    users: AllowedTenants
    services: AllowedTenants
    any_token_read: bool
    any_token_write: bool

    @staticmethod
    def from_spec(
        spec: Dict[str, Dict[str, Optional[List[Union[str, List[str]]]]]]
    ) -> Auth:
        def users(role: str) -> List[str]:
            return (
                list(set(ensure_flat_list(spec["users"].get(role))))
                if spec["users"]
                else []
            )

        def services(role: str) -> List[str]:
            return (
                list(set(ensure_flat_list(spec["services"].get(role))))
                if spec["services"]
                else []
            )

        return Auth(
            users=AllowedTenants(
                admins=users("admins"),
                writers=users("writers"),
                readers=users("readers"),
            ),
            services=AllowedTenants(
                admins=services("admins"),
                writers=services("writers"),
                readers=services("readers"),
            ),
            any_token_read=spec["anyToken"].get("read", False),
            any_token_write=spec["anyToken"].get("write", False),
        )

    def to_spec(self) -> dict:
        return {
            "users": {
                "admins": self.users.admins,
                "readers": self.users.readers,
                "writers": self.users.writers,
            },
            "services": {
                "admins": self.services.admins,
                "readers": self.services.readers,
                "writers": self.services.writers,
            },
            "anyToken": {"read": self.any_token_read, "write": self.any_token_write,},
        }



from __future__ import annotations

from dataclasses import dataclass
from typing import List, Optional
from uuid import UUID

from colorama import Fore

from clin.models.auth import Auth


@dataclass
class Subscription:
    id: Optional[UUID]
    owning_application: str
    event_types: List[str]
    consumer_group: str
    auth: Auth

    def __str__(self) -> str:
        prefix = f"subscription"
        if self.id:
            prefix += f" {Fore.BLUE}{self.id}{Fore.RESET}"
        return f"{prefix} ({Subscription.components_string(self.event_types, self.owning_application, self.consumer_group)})"

    @staticmethod
    def from_spec(spec: dict) -> Subscription:
        return Subscription(
            id=None,
            owning_application=spec["owningApplication"],
            event_types=spec["eventTypes"],
            consumer_group=spec["consumerGroup"],
            auth=Auth.from_spec(spec["auth"]),
        )

    @staticmethod
    def components_string(
        event_types: List[str], owning_application: str, consumer_group: str
    ):
        return f"event type(s): '{', '.join(event_types)}'; application: '{owning_application}'; consumer group: '{consumer_group}'"



from __future__ import annotations

from dataclasses import dataclass
from enum import Enum, unique
from typing import Optional, List, Dict

from colorama import Fore

from clin.models.auth import Auth


@unique
class Category(str, Enum):
    BUSINESS = "business"
    DATA = "data"
    UNDEFINED = "undefined"

    def __str__(self) -> str:
        return str(self.value)


@unique
class Audience(str, Enum):
    COMPONENT_INTERNAL = "component-internal"
    BUSINESS_UNIT_INTERNAL = "business-unit-internal"
    COMPANY_INTERNAL = "company-internal"
    EXTERNAL_PARTNER = "external-partner"
    EXTERNAL_PUBLIC = "external-public"

    def __str__(self) -> str:
        return str(self.value)


@dataclass
class Partitioning:
    @unique
    class Strategy(str, Enum):
        USER_DEFINED = "user_defined"
        HASH = "hash"
        RANDOM = "random"

        def __str__(self) -> str:
            return str(self.value)

    strategy: Strategy
    keys: Optional[List[str]]
    partition_count: int


@dataclass
class Cleanup:
    @unique
    class Policy(str, Enum):
        DELETE = "delete"
        COMPACT = "compact"

        def __str__(self) -> str:
            return str(self.value)

    policy: Policy
    retention_time_days: int


@dataclass
class Schema:
    @unique
    class Compatibility(str, Enum):
        NONE = "none"
        FORWARD = "forward"
        COMPATIBLE = "compatible"

        def __str__(self) -> str:
            return str(self.value)

    compatibility: Compatibility
    json_schema: Dict


@dataclass
class EventType:
    name: str
    category: Category
    owning_application: str
    audience: Audience
    partitioning: Partitioning
    cleanup: Cleanup
    schema: Schema
    auth: Auth

    def __str__(self) -> str:
        return f"event type {Fore.BLUE}{self.name}{Fore.RESET}"

    @staticmethod
    def from_spec(spec: dict) -> EventType:
        return EventType(
            name=spec["name"],
            category=Category(spec["category"]),
            owning_application=spec["owningApplication"],
            audience=Audience(spec["audience"]),
            partitioning=Partitioning(
                strategy=Partitioning.Strategy(spec["partitioning"]["strategy"]),
                keys=spec["partitioning"].get("keys"),
                partition_count=spec["partitioning"]["partitionCount"],
            ),
            cleanup=Cleanup(
                policy=Cleanup.Policy(spec["cleanup"]["policy"]),
                retention_time_days=spec["cleanup"].get(
                    "retentionTimeDays", 1
                ),  # 1 is default Nakadi value
            ),
            schema=Schema(
                compatibility=Schema.Compatibility(spec["schema"]["compatibility"]),
                json_schema=spec["schema"]["jsonSchema"],
            ),
            auth=Auth.from_spec(spec["auth"]),
        )

    def to_spec(self) -> dict:
        return {
            "name": self.name,
            "category": str(self.category),
            "owningApplication": self.owning_application,
            "audience": str(self.audience),
            "partitioning": {
                "strategy": str(self.partitioning.strategy),
                "keys": self.partitioning.keys,
                "partitionCount": self.partitioning.partition_count,
            },
            "cleanup": {
                "policy": str(self.cleanup.policy),
                "retentionTimeDays": self.cleanup.retention_time_days,
            },
            "schema": {
                "compatibility": str(self.schema.compatibility),
                "jsonSchema": self.schema.json_schema,
            },
            "auth": self.auth.to_spec() if self.auth else {},
        }



import logging
from dataclasses import dataclass
from itertools import chain
from pathlib import Path
from typing import List

from clin.yamlops import YamlLoader


@dataclass
class Process:
    id: str
    path: str
    kind: str
    spec: dict
    target: str


def calculate_scope(master: dict, base_path: Path, loader: YamlLoader) -> List[Process]:
    processes = master.get("process")
    if not processes:
        raise Exception("'process' section is not found")

    scope = []
    for proc in processes:
        proc_id = proc["id"]
        proc_paths = proc["paths"]

        sources = [base_path.joinpath(s).resolve(strict=False) for s in proc_paths]
        manifests_files = []
        for source in sources:
            if not source.exists():
                raise Exception(
                    f"Specified path 'f{source.absolute()}' is not found for process '{proc_id}'"
                )

            manifests_files += chain(source.glob("*.yml"), source.glob("*.yaml"))
            if not manifests_files:
                logging.warning(
                    "No manifests found for process '%s' in '%s'",
                    proc_id,
                    source.absolute(),
                )

        for manifest_file in manifests_files:
            yml = loader.load_yaml_from_file(manifest_file, proc.get("env", {}))

            kind = yml.get("kind")
            if not kind:
                raise Exception(
                    f"Manifest file '{manifest_file}' for process '{proc_id}' is malformed, required field `kind` is not found"
                )

            spec = yml.get("spec")
            if not spec:
                raise Exception(
                    f"Manifest file '{manifest_file}' for process '{proc_id}' is malformed, required field `spec` is not found"
                )

            scope.append(
                Process(
                    id=proc_id,
                    path=manifest_file,
                    kind=kind,
                    spec=spec,
                    target=proc["target"],
                )
            )
    return scope



import json
import logging
import sys

import yaml
from pygments import highlight
from pygments.formatter import Formatter
from pygments.formatters.other import NullFormatter
from pygments.formatters.terminal256 import TerminalTrueColorFormatter
from pygments.lexers.data import JsonLexer, YamlLexer

MS_IN_DAY = 24 * 60 * 60 * 1000


def walk(src, f):
    if isinstance(src, dict):
        for k, v in src.items():
            src[k] = walk(v, f)
        return src
    elif isinstance(src, list):
        for i, e in enumerate(src):
            src[i] = walk(e, f)
        return src
    else:
        return f(src)  # walk(f(src, f)) to enable recursive resolutions


def ensure_flat_list(val) -> list:
    if val is None:
        return []
    elif isinstance(val, list):
        return [item for sublist in val for item in ensure_flat_list(sublist)]
    else:
        return [val]


def configure_logging(verbose: bool):
    logging.basicConfig(level=logging.INFO, format="%(message)s", stream=sys.stdout)
    if verbose:
        logging.getLogger().setLevel(logging.DEBUG)
    logging.getLogger("requests").setLevel(logging.WARNING)
    logging.getLogger("urllib3").setLevel(logging.WARNING)


def pretty_yaml(val: dict, indentation: int = 0) -> str:
    raw = highlight(yaml.dump(val), YamlLexer(), getFormatter()).strip()
    return indent(raw, indentation)


def pretty_json(val: dict, indentation: int = 0) -> str:
    raw = highlight(json.dumps(val, indent=2), JsonLexer(), getFormatter())
    return indent(raw, indentation)


def indent(s: str, indentation: int) -> str:
    prepend = " " * indentation
    return "\n".join(prepend + l for l in s.splitlines())


def getFormatter() -> Formatter:
    return TerminalTrueColorFormatter() if sys.stdout.isatty() else NullFormatter()



import logging
from typing import Union, Optional

from colorama import Fore
from deepdiff import DeepDiff

from clin.config import AppConfig
from clin.models.event_type import EventType
from clin.models.subscription import Subscription
from clin.nakadi import (
    Nakadi,
    NakadiError,
    event_type_to_payload,
    subscription_to_payload,
)
from clin.utils import pretty_yaml, pretty_json

MODIFY_COLOR = Fore.MAGENTA
UP_TO_DATE_COLOR = Fore.GREEN
OUTPUT_INDENTATION = 4


class Processor:
    def __init__(
        self,
        config: AppConfig,
        token: Optional[str],
        execute: bool = False,
        show_diff: bool = False,
        show_payload: bool = False,
    ):
        self.nakadi = {
            env: Nakadi(conf.nakadi_url, token)
            for env, conf in config.environments.items()
        }

        self.token = token
        self.execute = execute
        self.show_diff = show_diff
        self.show_payload = show_payload

    def apply(self, env: str, kind: str, spec: dict):
        if kind == "event-type":
            self.apply_event_type(env, spec)
        elif kind == "subscription":
            self.apply_subscription(env, spec)
        else:
            raise ProcessingError(f"Invalid kind: {kind}")

    def apply_event_type(self, env: str, spec: dict):
        nakadi = self._get_nakadi(env)
        et = EventType.from_spec(spec)

        try:
            current = nakadi.get_event_type(et.name)
            if current:
                logging.debug("Found existing %s", et)
                diff = DeepDiff(current, et, ignore_order=True, report_repetition=True)
                if diff:
                    self._maybe_print_diff(et, diff)
                    self._maybe_print_payload(et)
                    self._update_event_type(nakadi, et)

                else:
                    logging.info(f"{UP_TO_DATE_COLOR}✔ Up to date:{Fore.RESET} %s", et)

            else:
                logging.debug("Not found existing %s", et)
                self._maybe_print_payload(et)
                self._create_event_type(nakadi, et)

        except NakadiError as err:
            raise ProcessingError(f"Can not process {et}: {err}") from err

    def apply_subscription(self, env: str, spec: dict):
        nakadi = self._get_nakadi(env)
        sub = Subscription.from_spec(spec)

        try:
            current = nakadi.get_subscription(
                sub.event_types, sub.owning_application, sub.consumer_group
            )
            if current:
                logging.debug("Found existing %s", current)
                sub.id = current.id
                diff = DeepDiff(current, sub, ignore_order=True, report_repetition=True)
                if diff:
                    self._maybe_print_diff(sub, diff)
                    self._maybe_print_payload(sub)
                    self._update_subscription(nakadi, sub)

                else:
                    logging.info(f"{UP_TO_DATE_COLOR}✔ Up to date:{Fore.RESET} %s", sub)

            else:
                logging.debug("Not found existing subscriptions matching: %s", sub)
                self._maybe_print_payload(sub)
                self._create_subscription(nakadi, sub)

        except NakadiError as err:
            raise ProcessingError(f"Can not process {sub}: {err}") from err

    def _get_nakadi(self, env: str):
        try:
            return self.nakadi[env]
        except KeyError:
            raise ProcessingError(f"Unknown environment: {env}")

    def _maybe_print_diff(self, entity: Union[EventType, Subscription], diff: DeepDiff):
        if self.show_diff:
            logging.info(
                f"{MODIFY_COLOR}⦿ Found %d changes:{Fore.RESET} %s\n%s",
                len(diff),
                entity,
                pretty_yaml(diff.to_dict(), indentation=OUTPUT_INDENTATION),
            )

    def _maybe_print_payload(self, entity: Union[EventType, Subscription]):
        if self.show_payload:
            if isinstance(entity, EventType):
                payload = event_type_to_payload(entity)
            elif isinstance(entity, Subscription):
                payload = subscription_to_payload(entity)
            else:
                logging.warning("Failed to print payload for %s", entity)
                return

            logging.info(
                f"{MODIFY_COLOR}⦿ Nakadi payload:{Fore.RESET} %s\n%s",
                entity,
                pretty_json(payload, indentation=OUTPUT_INDENTATION),
            )

    def _update_event_type(self, nakadi, et):
        if self.execute:
            nakadi.update_event_type(et)
            logging.info(f"{MODIFY_COLOR}⦿ Updated:{Fore.RESET} %s", et)
        else:
            logging.info(f"{MODIFY_COLOR}⦿ Will update:{Fore.RESET} %s", et)

    def _create_event_type(self, nakadi, et):
        if self.execute:
            nakadi.create_event_type(et)
            logging.info(f"{MODIFY_COLOR}⦿ Created:{Fore.RESET} %s", et)
        else:
            logging.info(f"{MODIFY_COLOR}⦿ Will create:{Fore.RESET} %s", et)

    def _update_subscription(self, nakadi, sub):
        if self.execute:
            nakadi.update_subscription(sub)
            logging.info(f"{MODIFY_COLOR}⦿ Updated:{Fore.RESET} %s", sub)
        else:
            logging.info(f"{MODIFY_COLOR}⦿ Will update:{Fore.RESET} %s", sub)

    def _create_subscription(self, nakadi, sub):
        if self.execute:
            nakadi.create_subscription(sub)
            logging.info(f"{MODIFY_COLOR}⦿ Created:{Fore.RESET} %s", sub)
        else:
            logging.info(f"{MODIFY_COLOR}⦿ Will create:{Fore.RESET} %s", sub)


class ProcessingError(Exception):
    def __init__(self, msg: str):
        self.msg = msg

    def __str__(self):
        return self.msg



import logging
import re
from pathlib import Path
from typing import List, Tuple

import yaml

from clin.utils import walk


class YamlLoader:
    def __init__(self, **kwargs):
        self._include_marker = kwargs.get("include_marker", "@@@")
        self._include_substitution = kwargs.get("include_substitution", "📁")
        self._variable_markers = kwargs.get("variable_markers", ("{{", "}}"))
        self._variable_substitutions = kwargs.get("variable_substitutions", ("👉", "👈"))

    def load_yaml_from_file(self, path: Path, env: dict) -> dict:
        yml = self._load_yaml_from_file(path, env, [])
        yml = self._resolve_variables(yml, env, path)
        return yml

    def _load_yaml_from_file(self, path: Path, env: dict, visited: List[Path]) -> dict:
        def resolve_include(src):
            if not isinstance(src, str) or not src.startswith(
                self._include_substitution
            ):
                return src
            include_path = path.parent.joinpath(src[1:])
            logging.debug(
                "Will include file %s to %s",
                include_path.absolute().resolve(),
                path.absolute().resolve(),
            )
            return self._load_yaml_from_file(include_path, env, visited + [path])

        if not path.exists():
            raise YamlFileNotFound(path)

        if not path.is_file():
            raise YamlNotReadableFile(path)

        if path in visited:
            raise YamlCycleReferenceError(path)

        content = (
            path.read_text()
            .replace(self._include_marker, self._include_substitution)
            .replace(self._variable_markers[0], self._variable_substitutions[0])
            .replace(self._variable_markers[1], self._variable_substitutions[1])
        )

        yml = yaml.full_load(content)
        return walk(yml, resolve_include)

    def _resolve_variables(self, process, env, path):
        vars_re = re.compile(
            rf"{self._variable_substitutions[0]}(.*?){self._variable_substitutions[1]}"
        )

        def loop(s):
            if not isinstance(s, str):
                return s

            found_vars = re.findall(vars_re, s)
            if not found_vars:
                return s

            for found_var in found_vars:
                if found_var not in env:
                    raise YamlUnknownVariableError(path, found_var)

                substitution = env.get(found_var)
                if substitution is None:
                    raise YamlUndefinedVariableError(path, found_var)

                if (
                    s
                    == f"{self._variable_substitutions[0]}{found_var}{self._variable_substitutions[1]}"
                ):
                    s = env.get(found_var)
                else:
                    if type(substitution) in (list, dict):
                        raise YamlIncorrectSubstitutionError(
                            path, found_var, substitution
                        )
                    s = s.replace(
                        f"{self._variable_substitutions[0]}{found_var}{self._variable_substitutions[1]}",
                        str(substitution),
                    )
            return s

        return walk(process, loop)


class YamlError(Exception):
    def __init__(self, file: Path):
        self.file = file


class YamlFileNotFound(YamlError):
    def __str__(self):
        return f"File {self.file.absolute()} is not found"


class YamlNotReadableFile(YamlError):
    def __str__(self):
        return f"File {self.file.absolute()} can not be read"


class YamlCycleReferenceError(YamlError):
    def __str__(self):
        return f"File {self.file.absolute()} is referenced in cycle"


class YamlUnknownVariableError(YamlError):
    def __init__(self, file: Path, variable: str):
        super(YamlUnknownVariableError, self).__init__(file)
        self._variable = variable

    def __str__(self):
        return f"Variable {self._variable} (in {self.file}) is not found in provided environment"


class YamlUndefinedVariableError(YamlError):
    def __init__(self, file: Path, variable: str):
        super(YamlUndefinedVariableError, self).__init__(file)
        self._variable = variable

    def __str__(self):
        return f"Variable {self._variable} (in {self.file}) is not defined (i.e. `VAR: `), explicit value required (i.e. `VAR: []`)"


class YamlIncorrectSubstitutionError(YamlError):
    def __init__(self, file: Path, variable: str, value: dict):
        super(YamlIncorrectSubstitutionError, self).__init__(file)
        self._variable = variable
        self._value = value

    def __str__(self):
        return f"Variable {self._variable} can not be resolved in given context by {self._value} in {self.file}"


class YamlInvalidFormatError(YamlError):
    def __init__(self, file: Path, message: str):
        super(file)
        self.message = message

    def __str__(self):
        return f"{self.message} in {self.file.absolute()}"


def load_yaml(file_path: Path, loader: YamlLoader, env: dict) -> dict:
    return loader.load_yaml_from_file(file_path, env)


def load_manifest(file_path: Path, loader: YamlLoader, env: dict) -> Tuple[str, dict]:
    manifest = load_yaml(file_path, loader, env)

    if "kind" not in manifest:
        raise YamlInvalidFormatError(file_path, f"Required field `kind` not found")
    if "spec" not in manifest:
        raise YamlInvalidFormatError(file_path, f"Required field `spec` not found")

    if manifest["kind"] not in ["event-type", "subscription"]:
        raise YamlInvalidFormatError(
            file_path, f"Unknown manifest of kind '{manifest['kind']}'"
        )

    return manifest["kind"], manifest["spec"]
